{
    "$graph": [
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 1,
                    "ramMin": 8000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "brianyee/umi_tools:1.0.0"
                }
            ],
            "baseCommand": [
                "umi_tools",
                "extract"
            ],
            "arguments": [
                "--random-seed",
                "1"
            ],
            "inputs": [
                {
                    "type": "string",
                    "default": "NNNNNNNNNN",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--bc-pattern"
                    },
                    "doc": "10 nt randomer",
                    "id": "#demux_se.cwl/bc_pattern"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#demux_se.cwl/dataset"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--log",
                        "valueFrom": "${\n  if (inputs.log == \"\") {\n    return inputs.dataset + \".\" + inputs.reads.name + \".---.--.metrics\";\n  }\n  else {\n    return inputs.log;\n  }\n}\n"
                    },
                    "id": "#demux_se.cwl/log"
                },
                {
                    "type": {
                        "type": "record",
                        "fields": [
                            {
                                "type": "string",
                                "name": "#demux_se.cwl/reads/name"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 1,
                                    "prefix": "--stdin"
                                },
                                "name": "#demux_se.cwl/reads/read1"
                            }
                        ]
                    },
                    "id": "#demux_se.cwl/reads"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--stdout",
                        "valueFrom": "${\n  if (inputs.stdout == \"\") {\n    return inputs.dataset + \".\" + inputs.reads.name + \".umi.r1.fq\";\n  }\n  else {\n    return inputs.stdout;\n  }\n}\n"
                    },
                    "id": "#demux_se.cwl/stdout"
                }
            ],
            "doc": "Extract UMI barcode from a read and add it to the read name, leaving\nany sample barcode in place. Can deal with paired end reads and UMIs\nsplit across the paired ends. For eCLIP single-end processing, this step just\ntrims the first 10 bases, but named as such to match the demux_pe step.\n\n  Usage: umi_tools extract --bc-pattern=[PATTERN] -L extract.log [OPTIONS]\n",
            "id": "#demux_se.cwl",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).umi.r1.fq"
                    },
                    "id": "#demux_se.cwl/demuxedAfwd"
                },
                {
                    "type": "string",
                    "outputBinding": {
                        "loadContents": true,
                        "outputEval": "$(inputs.reads.name)"
                    },
                    "doc": "just passes output name string to output to match with PE demux",
                    "id": "#demux_se.cwl/name"
                },
                {
                    "type": "string",
                    "outputBinding": {
                        "loadContents": true,
                        "outputEval": "$(inputs.dataset)"
                    },
                    "doc": "just passes output dataset string to output to match with PE demux",
                    "id": "#demux_se.cwl/output_dataset"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.dataset).$(inputs.reads.name).---.--.metrics"
                    },
                    "label": "",
                    "doc": "demuxed se metrics",
                    "id": "#demux_se.cwl/output_demuxedsingleend_metrics"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "baseCommand": [
                "gzip"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "id": "#gzip.cwl/input"
                },
                {
                    "type": "boolean",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "default": true,
                    "id": "#gzip.cwl/stdout"
                }
            ],
            "stdout": "$(inputs.input.basename).gz",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input.basename).gz"
                    },
                    "id": "#gzip.cwl/gzipped"
                }
            ],
            "id": "#gzip.cwl"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#main/dataset"
                },
                {
                    "type": {
                        "type": "record",
                        "fields": [
                            {
                                "type": "string",
                                "name": "#main/read/name"
                            },
                            {
                                "type": "File",
                                "name": "#main/read/read1"
                            }
                        ]
                    },
                    "id": "#main/read"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/gzip_demux/gzipped",
                    "id": "#main/A_output_demuxed_read1"
                },
                {
                    "type": "string",
                    "outputSource": "#main/AB_demux/output_dataset",
                    "id": "#main/dataset_name"
                },
                {
                    "type": "string",
                    "outputSource": "#main/AB_demux/name",
                    "id": "#main/read_name"
                }
            ],
            "steps": [
                {
                    "run": "#demux_se.cwl",
                    "in": [
                        {
                            "source": "#main/dataset",
                            "id": "#main/AB_demux/dataset"
                        },
                        {
                            "source": "#main/read",
                            "id": "#main/AB_demux/reads"
                        }
                    ],
                    "out": [
                        "#main/AB_demux/demuxedAfwd",
                        "#main/AB_demux/output_demuxedsingleend_metrics",
                        "#main/AB_demux/output_dataset",
                        "#main/AB_demux/name"
                    ],
                    "id": "#main/AB_demux"
                },
                {
                    "run": "#gzip.cwl",
                    "in": [
                        {
                            "source": "#main/AB_demux/demuxedAfwd",
                            "id": "#main/gzip_demux/input"
                        }
                    ],
                    "out": [
                        "#main/gzip_demux/gzipped"
                    ],
                    "id": "#main/gzip_demux"
                }
            ],
            "doc": "This workflow takes in single-end reads, and performs the following steps in order:\ndemux_se.cwl (does not actually demux for single end, but mirrors the paired-end processing protocol)\n",
            "id": "#main"
        }
    ],
    "cwlVersion": "v1.0"
}
